@@ -12,7 +12,6 @@
 #' \item{coef}{OLS coefficient of X ~ Z (reported if Z is not binary)}
 #' \item{se}{Standard error of OLS coefficient (reported if Z is not binary)}
 #' \item{p.val}{p-value of the independence of Z and X (Fisher's test if both are binary, logistic regression if Z is binary, linear regression if Z is continuous)}
-#' \item{x.sd}{sample standard deviations of X}
 #' \item{stand.diff}{Standardized difference (reported if Z is binary)}
 #' \item{bias.ratio}{Bias ratio}
 #' \item{bias.amplify}{Amplification of bias ratio}
@@ -25,39 +24,50 @@
 #' \itemize{
 #' \item{Baiocchi, M., Cheng, J., & Small, D. S. (2014). Instrumental variable methods for causal inference. Statistics in Medicine, 33(13), 2297-2340.}
 #' \item{Jackson, J. W., & Swanson, S. A. (2015). Toward a clearer portrayal of confounding bias in instrumental variable applications. Epidemiology, 26(4), 498.}
-#' \item{Zhao, Q. & Small, D. S. Graphical diagnosis of confounding bias in instrumental variable analysis. Epidemiology, forthcoming.}
-#'}
+#' \item{Zhao, Q., & Small, D. S. (2018). Graphical diagnosis of confounding bias in instrumental variable analysis. Epidemiology, 29(4), e29--e31.}
+#' }
 #'
 #' @importFrom stats fisher.test
 #' @importFrom stats glm
-#' @importFrom stats sd
 #'
 #' @export
 #'
 #' @examples
 #' n <- 10000
 #' Z <- rbinom(n, 1, 0.5)
-#' X <- data.frame(matrix(rbinom(n * 5, 1, 0.5), n))
+#' X <- data.frame(matrix(c(rnorm(n), rbinom(n * 5, 1, 0.5)), n))
 #' D <- rbinom(n, 1, plogis(Z + X[, 1] + X[, 2] + X[, 3]))
 #' Y <- D + X[, 1] + X[, 2] + rnorm(n)
-#' output <- iv.diagnosis(Y, D, Z, X)
-#' print(output)
+#' print(output <- iv.diagnosis(Y, D, Z, X))
+#' iv.diagnosis.plot(output)
+#'
+#' Z <- rnorm(n)
+#' D <- rbinom(n, 1, plogis(Z + X[, 1] + X[, 2] + X[, 3]))
+#' Y <- D + X[, 1] + X[, 2] + rnorm(n)
+#' print(output <- iv.diagnosis(Y, D, Z, X)) ## stand.diff is not reported
 #' iv.diagnosis.plot(output)
 #'
 iv.diagnosis <- function(Y, D, Z, X) {
 
     if (length(dim(X)) == 2) { ## if x is a matrix/data.frame
-        return(t(apply(X, 2, function(X) unlist(iv.diagnosis(Y, D, Z, X)))))
+        output <- data.frame(t(sapply(1:ncol(X), function(j) unlist(iv.diagnosis(Y, D, Z, X[, j])))))
+        rownames(output) <- colnames(X)
+        return(output)
     }
 
     if (length(unique(X)) == 2 & length(unique(Z)) == 2) {
         p.val <- fisher.test(X, Z)$p.value
-        stand.diff <- (mean(X[Z==1])-mean(X[Z==0]))/(sqrt((var(X[Z==1])+var(X[Z==0]))/2));
     } else if (length(unique(X)) == 2) {
         p.val <- summary(glm(X ~ Z, family = "binomial"))$coefficients[2, 4]
     } else {
         p.val <- summary(lm(X ~ Z))$coefficients[2, 4]
     }
+    if (length(unique(Z)) == 2) {
+        if (!all.equal(sort(unique(Z)), c(0, 1))) {
+            stop("Please convert Z to {0, 1}.")
+        }
+        stand.diff <- (mean(X[Z==1])-mean(X[Z==0]))/(sqrt((var(X[Z==1])+var(X[Z==0]))/2))
+    }
     prev.diff.ratio <- (lm(X ~ Z)$coef[2])/(lm(X ~ D)$coef[2]);
     bias.ratio <- prev.diff.ratio / (lm(D ~ Z)$coef[2])
     bias.amplify <- lm(Y ~ X + D)$coef[2]
@@ -68,21 +78,24 @@ iv.diagnosis <- function(Y, D, Z, X) {
     names(bias.ols) <- NULL
     names(bias.2sls) <- NULL
     ## stand.diff.high.treatment <- (mean(x[d==1])-mean(x[d==0]))/sqrt((var(x[d==1])+var(x[d==0]))/2);
-    if (length(unique(Z)) == 2) {
+    if (length(unique(X)) == 2 & length(unique(Z)) == 2) {
         output <- list(x.mean1 = mean(X[Z == 1]),
                        x.mean0 = mean(X[Z == 0]),
+                       coef = NA,
+                       se = NA,
                        p.val = p.val,
                        stand.diff = stand.diff,
-                       x.sd = sd(X),
                        bias.ratio = bias.ratio,
                        bias.amplify = bias.amplify,
                        bias.ols = bias.ols,
                        bias.2sls = bias.2sls)
     } else {
-        output <- list(coef = lm(X ~ Z)$coef[2],
+        output <- list(x.mean1 = NA,
+                       x.mean0 = NA,
+                       coef = unname(lm(X ~ Z)$coef[2]),
                        se = summary(lm(X ~ Z))$coefficients[2, 2],
                        p.val = p.val,
-                       x.sd = sd(X),
+                       stand.diff = NA,
                        bias.ratio = bias.ratio,
                        bias.amplify = bias.amplify,
                        bias.ols = bias.ols,
@@ -94,7 +107,6 @@ iv.diagnosis <- function(Y, D, Z, X) {
 #' @describeIn iv.diagnosis IV diagnostic plot
 #'
 #' @param output Output from \code{iv.diagnosis}.
-#' @param order.by Order the bars by bias amplifying factor (variance of the outcome explained by each covariate), bias of the OLS estimate, or bias of the 2SLS estimate.
 #' @param bias.ratio Add bias ratios (text) to the plot?
 #' @param base_size size of the axis labels
 #' @param text_size size of the text (bias ratios)
@@ -103,16 +115,10 @@ iv.diagnosis <- function(Y, D, Z, X) {
 #' @import reshape2
 #' @import ggplot2
 #'
-iv.diagnosis.plot<- function(output, order.by = c("bias.amplify", "ols.bias", "2sls.bias"), bias.ratio = TRUE,  base_size = 15, text_size = 5) {
-
-    order.by <- match.arg(order.by, c("bias.amplify", "ols.bias", "2sls.bias"))
+iv.diagnosis.plot<- function(output, bias.ratio = TRUE, base_size = 15, text_size = 5) {
 
     output <- data.frame(output)
-    o <- switch(order.by,
-                bias.amplify = order(abs(output$bias.amplify) * output$x.sd),
-                bias.ols = order(abs(output$bias.ols)),
-                bias.2sls = order(abs(output$bias.2sls)))
-    output$var <- factor(rownames(output), rownames(output)[o])
+    output$var <- factor(rownames(output), rownames(output)[order(abs(output$bias.ols))])
     rownames(output) <- NULL
     df <- data.frame(output[, c("var", "bias.ols", "bias.2sls")])
     df <- melt(df, id.vars = "var")
@@ -120,7 +126,8 @@ iv.diagnosis.plot<- function(output, order.by = c("bias.amplify", "ols.bias", "2
     levels(df$method) <- c("ols", "2sls")
     df$method <- factor(df$method, levels = c("2sls", "ols"))
 
-    p <- ggplot() + geom_bar(aes_string(x = "var", y = "bias", fill = "method", linetype = "method"), stat = "identity", color = "black", position=position_dodge(), data = df) + coord_flip() + theme_bw(base_size) + ylim(range(df$bias) * 1.2) + xlab("") + scale_fill_discrete(breaks = c("ols","2sls")) + scale_linetype_discrete(breaks = c("ols","2sls"))
+    p <- ggplot() + geom_bar(aes_string(x = "var", y = "bias", fill = "method", linetype = "method"), stat = "identity", color = "black", position=position_dodge(), data = df) + coord_flip() + theme_bw(base_size) + xlab("") + scale_fill_discrete(breaks = c("ols","2sls")) + scale_linetype_discrete(breaks = c("ols","2sls")) + expand_limits(y=c(0, range(df$bias) * 1.2))
+
     if (bias.ratio) {
         p <- p + geom_text(mapping = aes(y = max(df$bias) * 1.1, x = var, label = round(bias.ratio, 2)), data = data.frame(output), size = text_size)
     }
